<?php


namespace Controller;

use Service\Issue as IssueService;
use Traits\TResponse;
use Util\InstanceProvider;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class Issue
{
    use TResponse;

    /**
     * @var IssueService
     */
    private $issueService;

    public function __construct(IssueService $issueService) {
        $this->issueService = $issueService;
    }

    public function findActionsOfExits(Request $request, Response $response)
    {
        try {
            $issueId = $request->getParameter('issueId');
            $actions = $this->issueService->getActionsOfExits($issueId);
            $this->sendResponse($actions, 200);
        }catch (\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveSendOutComment(Request $request, Response $response){
        $params = $request->getParameters();
        try {
            $description = $params['DESCRIPTION'];
            if(!empty($params['DOCUMENT'])) {
                InstanceProvider::getFileUploaderServe()->uploadFilesInRow($params, 'DOCUMENT', 'PmDevOps/Task');
                $document = $params['DOCUMENT'];
            } else {
                $document = array();
            }

            $taskTypeTarget = $params['TASK_TYPE_TARGET'];
            $commentTypeId = $params['COMMENT_TYPE_ID'];
            $issueId = $params['ISSUE_ID'];
            $specialFunction = empty($params['SPECIAL_FUNCTION']) ? null : $params['SPECIAL_FUNCTION'];

            $this->issueService->sendOutComment($issueId, $document, $taskTypeTarget, $commentTypeId, $description,$specialFunction);
        }
        catch(\Exception $e){
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
    public function clientRefuse(Request $request, Response $response){
        $params = $request->getParameters();
        try {
            $description = $params['DESCRIPTION'];
            if(!empty($params['DOCUMENT'])) {
                InstanceProvider::getFileUploaderServe()->uploadFilesInRow($params, 'DOCUMENT', 'PmDevOps/Task');
                $document = $params['DOCUMENT'];
            } else {
                $document = array();
            }

            $commentTypeId = $params['COMMENT_TYPE_ID'];
            $issueId = $params['ISSUE_ID'];

            $this->issueService->clientRefuse($issueId,$commentTypeId,$description, $document);
        }
        catch(\Exception $e){
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
    public function closeIssue(Request $request, Response $response){
        $params = $request->getParameters();
        try {
            $issueId = $params['ISSUE_ID'];

            $this->issueService->closeIssue($issueId);
        }
        catch(\Exception $e){
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudgetSave(Request $request, Response $response){
        $params = $request->getParameters();
        try {
            $issueId = $params['CALL_RELATIONSHIP'];
            $callType = $params['CALL_TYPE'];
            $data = $params['DATA'];
            $dataMaintenance = $params['DATA_MAINTENANCE'];

            $this->issueService->issueBudgetSave($data);
            $this->issueService->saveMaintenanceCost($dataMaintenance);

            $this->sendResponse([], 200);
        }
        catch(\Exception $e){
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudgetInstallmentsSave(Request $request, Response $response){
        $params = $request->getParameters();
        try {
            $issueId = $params['CALL_RELATIONSHIP'];
            $data = $params['DATA'];

            $this->issueService->issueBudgetPlotsSave($data);

            $this->sendResponse([], 200);
        }
        catch(\Exception $e){
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function maintenanceCost(Request $request, Response $response) {
        try{
            $params = $request->getParameters();
            $issueId = $params['CALL_RELATIONSHIP'];

            $budget = $this->issueService->findMaintenanceCost($issueId);
            $data = array(
                "data" => $budget,
                "pagination" => array(
                    "limit" => 500000,
                    "page" => 1,
                    "total" => count($budget))
            );
            $this->sendResponse($data, 200);
        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudget(Request $request, Response $response) {
        try{
            $conditions = $request->getParameters();
            $budget = [];
            $issueId = empty($conditions['ISSUE']) ? null : $conditions['ISSUE'];
            $nameProposal = empty($conditions['NAME_PROPOSAL']) ? null : $conditions['NAME_PROPOSAL'];
            $namePercent = empty($conditions['NAME_PERCENT']) ? null : $conditions['NAME_PERCENT'];
            $nameDiscount = empty($conditions['NAME_DISCOUNT']) ? null : $conditions['NAME_DISCOUNT'];
            if($issueId){
                $budget = InstanceProvider::getService()->getIssue()->findIssueBudget($issueId,$nameProposal,
                    $nameDiscount,$namePercent);
            }
            foreach ($budget as &$item) {
                $item['PROPOSAL_VALUE'] = intval($item['PROPOSAL_VALUE']);
            }
            $data = array(
                "data" => $budget,
                "pagination" => array(
                    "limit" => 500000,
                    "page" => 1,
                    "total" => count($budget))
            );
            $this->sendResponse($data, 200);
        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function issueBudgetPlots(Request $request, Response $response) {
        try{
            $conditions = $request->getParameters();
            $budget = [];
            $issueId = empty($conditions['ISSUE']) ? null : $conditions['ISSUE'];
            $namePlots = empty($conditions['NAME_PLOTS']) ? null : $conditions['NAME_PLOTS'];
            if($issueId){
                $budget = InstanceProvider::getService()->getIssue()->findIssueBudgetPlots($issueId, $namePlots);
            }
            foreach ($budget as &$item) {
                $item['PROPOSAL_VALUE'] = intval($item['PROPOSAL_VALUE']);
                $item['VALUE_PLOTS'] = intval($item['VALUE_PLOTS']);
                $item['DAYS_PAYMENT'] = intval($item['DAYS_PAYMENT']);
                $item['ORDER_PLOTS_NUM'] = intval($item['ORDER_PLOTS_NUM']);
            }
            $data = array(
                "data" => $budget,
                "pagination" => array(
                    "limit" => 500000,
                    "page" => 1,
                    "total" => count($budget))
            );
            $this->sendResponse($data, 200);
        } catch(\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
}
