<?php

namespace Service\Integration;

use Util\Utilities;

class GoogleDrive
{
    //"login": "javamailteste1@gmail.com",
    //"senha": "teknisateste"
    public $client;

    public function __construct()
    {
        $client = new \Google_Client();
        $client->setApplicationName('TeknisaDocs');
        $client->setScopes([\Google\Service\Docs::DOCUMENTS,\Google\Service\Drive::DRIVE]);
        $client->setAccessType('offline');
        $client->setAuthConfig('../config/credentialsIAM.json');
        $client->setPrompt('select_account consent');
        $this->client = $client;
    }

    public function saveDoc($file, $fileId){
        $client = $this->client;
        $driveService = new \Google\Service\Drive($client);

        $response = $driveService->files->get($fileId, array(
            [
                'type' => null,
                'location' => null,
                'alt' => 'media',
            ],
            'fields' => '*'
        ));

        $extensions = explode('.', $file['SIMPLE_FILE_NAME']);
        $nameBase   = $extensions[0];

        $googleFileName = explode('.', $response->name);
        $extensionFinal = array_pop($googleFileName);

        $fileNameSave = $file['CALL_DIR'] . DIRECTORY_SEPARATOR . $nameBase . '.' . $extensionFinal;
        $var = file_get_contents($response->webContentLink);
        file_put_contents($fileNameSave, $var);

        $driveService->files->delete($fileId);
        $fileExtension = Utilities::getExtension($file['SPEC_FILE_NAME']);
        if(strtolower($fileNameSave) != strtolower($file['SPEC_FILE_NAME']) && $fileExtension != 'docx') {
            unlink($file['SPEC_FILE_NAME']);
        }
    }

    public function deleteFileDrive($fileId){
        $client = $this->client;
        $driveService = new \Google\Service\Drive($client);
        $driveService->files->delete($fileId);
    }

    public function uploadDoc($file){
        $client = $this->client;
        $driveService = new \Google\Service\Drive($client);

        if($file['IS_DEFAULT_FILE']) {
            $data = @file_get_contents($file['DEFAULT_FILE_NAME']);
        } else {
            $data = @file_get_contents($file['SPEC_FILE_NAME']);
        }

        if(empty($data)) {
            throw New \Exception('Documento de especificação não encontrado.');
        }

        $mimeType = Utilities::getTypeFile($file['SPEC_FILE_NAME']);
        $document = $this->CreateDoc($file['SIMPLE_FILE_NAME'], $mimeType, $driveService, $data);
        $fileId = $document->id;
        $OptParams = array(
            'fields' => '*'
        );

        $this->setPermissionAnyineWriter($driveService, $fileId, $OptParams);
        $urlDocGoogle = 'https://docs.google.com/document/d/' . $fileId . '/edit';
        return array('url' => $urlDocGoogle, 'id' => $fileId);
    }

    /**
     * @param string $nameBase
     * @param string $mimeType
     * @param \Google\Service\Drive $driveService
     * @param string $data
     * @return \Google\Service\Drive\DriveFile
     */
    public function CreateDoc(string $nameBase, string $mimeType, \Google\Service\Drive $driveService, string $data): \Google\Service\Drive\DriveFile
    {
        $fileMetadata = new \Google\Service\Drive\DriveFile(array(
            'name' => $nameBase));
        $fileMetadata->setMimeType($mimeType);
        $document = $driveService->files->create($fileMetadata, array([
            'mimeType' => $mimeType,
            'type' => "string",
            'location' => null,
            'uploadType' => 'multipart',
            'fields' => '*'
        ],
            'data' => $data));
        return $document;
    }

    /**
     * @param \Google\Service\Drive $driveService
     * @param string $fileId
     * @param array $OptParams
     */
    public function setPermissionAnyineWriter(\Google\Service\Drive $driveService, string $fileId, array $OptParams): void
    {
        $reponse = $driveService->files->get($fileId, $OptParams);
        foreach ($reponse->getPermissions() as $PermissionValue) {
            $updatedPerm = new \Google\Service\Drive\Permission(array(
                'type' => 'anyone',
                'role' => "writer",
                'additionalRoles' => [],
                'withLink' => true
            ));
            $driveService->permissions->create($fileId, $updatedPerm, array('fields' => 'id', 'supportsTeamDrives' => true));
        }
    }
}
