<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Attachment extends \Doctrine\ORM\EntityRepository {

	public function getDataSetForEditTaskAttachments($taskId) {
		$query =<<<SQL
			SELECT ATT.FILE_NAME, ATT.ID, ATT.LABEL, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT, BU.NAME AS CREATED_BY
			FROM 		ATTACHMENT ATT
			INNER JOIN 	RESOURCE_T BU ON(BU.ID = ATT.CREATED_BY)
			WHERE ATT.TASK = :ID_TASK
SQL;
		$params = array(
			'ID_TASK'  => $taskId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getDataSetForEditAttachment($id, $entity, $conditions) {
		$query =<<<SQL
			SELECT *
			FROM (SELECT ATT.FILE_NAME, ATT.ID, ATT.LABEL, TASK.NAME, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				   RES.NAME AS CREATED_BY, ATT.$entity
				FROM ATTACHMENT ATT
					INNER JOIN RESOURCE_T RES ON (RES.ID = ATT.CREATED_BY)	
					LEFT JOIN TASK ON (TASK.ID = ATT.TASK)
					LEFT JOIN COMMENTS C ON (ATT.COMMENTS = C.ID)
	        	WHERE (ATT.$entity = :ID OR ('$entity' = 'task' AND C.TASK =:ID))
						AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)
                        )
			 WHERE 1 = 1
	   $conditions
SQL;
		$params = array(
			'ID'  => $id,
			'TR_TYPE_ISSUE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
			'TR_TYPE_END_START' => Entities\TaskRelationship::END_START_TYPE,
			'TR_TYPE_PRECURSOR' => Entities\TaskRelationship::PRECURSOR_TYPE
		);
		$connection = $this->getEntityManager()->getConnection();
		return $connection->fetchAll($query, $params);
	}

    public function getAttachmentFromTask($id, $conditions,$isIssue = false) {
        $queryNotIssue = '';
        if($isIssue){
            $queryNotIssue = <<<SQL

         UNION

            SELECT NVL(TASK.NAME,CTA.NAME) as TASK_NAME, ATT.FILE_NAME, ATT.ID, ATT.LABEL, TASK.NAME, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				   RES.NAME AS CREATED_BY, ATT.TASK
			  FROM ATTACHMENT ATT
			  JOIN RESOURCE_T RES ON (RES.ID = ATT.CREATED_BY)
	     LEFT JOIN TASK ON (TASK.ID = ATT.TASK)
		 LEFT JOIN COMMENTS C ON (ATT.COMMENTS = C.ID)
         LEFT JOIN TASK CTA ON (CTA.ID = C.TASK)
	         WHERE (ATT.TASK = :ID OR  C.TASK =:ID)
			   AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)
SQL;

        }

		$query =<<<SQL
        SELECT *
        FROM (
            SELECT NVL(TASK.NAME,CTA.NAME) as TASK_NAME, ATT.FILE_NAME, ATT.ID, ATT.LABEL,TASK.NAME, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				   RES.NAME AS CREATED_BY, ATT.TASK
			  FROM ATTACHMENT ATT
			  JOIN RESOURCE_T RES ON (RES.ID = ATT.CREATED_BY)
			  JOIN TASK_RELATIONSHIP	TAR ON (TAR.TASK_RELATIONSHIP = :ID AND TAR.TYPE = :TYPE_FIRST AND TAR.TASK = ATT.TASK )
		 LEFT JOIN COMMENTS C ON (ATT.COMMENTS = C.ID)
         LEFT JOIN TASK ON (TASK.ID = TAR.TASK)
         LEFT JOIN TASK CTA ON (CTA.ID = C.TASK)
	         WHERE (C.TASK = TAR.TASK OR TAR.TASK_RELATIONSHIP = :ID)
			   AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)

           $queryNotIssue
          )
	   $conditions
SQL;
		$params = array(
			'ID'  => $id,
            'TYPE_FIRST' => Entities\TaskRelationship::FIRST_TYPE,
		);
		$connection = $this->getEntityManager()->getConnection();
		return $connection->fetchAll($query, $params);
	}
	public function getDataAttachmentMeeting($id) {
		$query =<<<SQL
		  SELECT  ATT.FILE_NAME, ATT.ID, ATT.LABEL, TASK.NAME, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				  RES.NAME AS CREATED_BY
		  FROM 			ATTACHMENT ATT
		  INNER JOIN 	RESOURCE_T RES 	ON (RES.ID = ATT.CREATED_BY)
		  LEFT JOIN 	TASK 			ON (TASK.ID = ATT.TASK)
		  WHERE (ATT.PROJECT IN (SELECT PRO.ID FROM PROJECT PRO WHERE PRO.PARENT = :ID OR PRO.ID = :ID))
		  AND 	(TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)
SQL;
		$params = array(
			'ID'  => $id
		);
		$connection = $this->getEntityManager()->getConnection();
		return $connection->fetchAll($query, $params);

	}

	public function getAttachmentSpecification($issueId) {
		$query =<<<SQL
		  SELECT * FROM ( SELECT ATT.ID
                            FROM ATTACHMENT ATT
                            JOIN COMMENTS COM ON (COM.ID = ATT.COMMENTS)
                            JOIN COMMENT_TYPE COT ON (COT.ID  = COM.COMMENT_TYPE_ID AND COT.CONTROL = :TYPE_SPECIFICATION)
                           WHERE COM.TASK = :ISSUE_ID
                        ORDER BY COM.ID DESC)
        WHERE ROWNUM = 1
SQL;
		$params = array(
			'ISSUE_ID'  => $issueId,
			'TYPE_SPECIFICATION'  => Entities\CommentType::TYPE_SPECIFICATION,

		);
		$connection = $this->getEntityManager()->getConnection();
        $result = $connection->fetchAll($query, $params);
        return empty($result) ? null : $result[0]['ID'];
	}

    public function getTaskAttachmentsExceptUploadedBy($issueId,$fileServerUrl,$filter = null) {
        $query = <<<SQL
		SELECT '$fileServerUrl'||ATT.FILE_NAME AS FILENAME, ATT.ID, ATT.LABEL, TASK.NAME, ATT.TYPE_FILE AS FILE_TYPE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
               ATT.CREATED_AT AS CREATED_AT_DATETIME,
			   RES.NAME AS CREATED_BY, C.ID AS COMMENT_ID
		  FROM ATTACHMENT ATT
	      JOIN RESOURCE_T RES 	ON (RES.ID = ATT.CREATED_BY)
	 LEFT JOIN TASK ON (TASK.ID = ATT.TASK)
	 LEFT JOIN COMMENTS C ON (ATT.COMMENTS = C.ID)
     LEFT JOIN TASK CTA ON (CTA.ID = C.TASK)
	     WHERE (ATT.TASK = :ID OR  C.TASK =:ID) $filter
		   AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)
	  ORDER BY ATT.CREATED_AT
SQL;

        $params = array(
            'ID' => $issueId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getIssueById($id,$fileServerUrl) {
        $query = <<<SQL
		  SELECT  '$fileServerUrl'||ATT.FILE_NAME AS FILENAME, ATT.ID, ATT.LABEL,
		          ATT.TYPE_FILE AS FILE_TYPE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				  RES.NAME AS CREATED_BY, ATT.SIZE_FILE
		  FROM 	  ATTACHMENT ATT
		  JOIN 	  RESOURCE_T RES 	ON (RES.ID = ATT.CREATED_BY)
		  WHERE   ATT.ID = :ID
		 ORDER BY ATT.CREATED_AT
SQL;

        $params = array(
            'ID' => $id
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getDeployMigrationsScripts($deployId, $filters = '') {
        $query = <<<SQL
        SELECT *
          FROM (SELECT A.ID, A.LABEL, A.TYPE_FILE,
                     A.CREATED_AT, CO.NMOPERADOR AS CREATED_BY,
                     A.UPDATED_AT, UO.NMOPERADOR AS UPDATED_BY
                FROM ATTACHMENT A
           LEFT JOIN RESOURCE_T CR ON A.CREATED_BY = CR.ID
           LEFT JOIN OPERADOR CO   ON CR.EXTERNAL_CODE = CO.CDOPERADOR
           LEFT JOIN RESOURCE_T UR ON A.UPDATED_BY = UR.ID
           LEFT JOIN OPERADOR UO   ON UR.EXTERNAL_CODE = UO.CDOPERADOR
               WHERE A.DEPLOY = :DEPLOY_ID
                 AND A.LABEL LIKE ('%.sql'))
         WHERE 1 = 1
          $filters
SQL;

        $params = array (
            'DEPLOY_ID' => $deployId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getDeployMigrationsScriptsBySubRelease($subReleaseId, $filters = '')
    {
        $query = <<<SQL
        SELECT *
          FROM (SELECT A.ID, A.LABEL, A.TYPE_FILE,
                     A.CREATED_AT, CO.NMOPERADOR AS CREATED_BY,
                     A.UPDATED_AT, UO.NMOPERADOR AS UPDATED_BY
                FROM ATTACHMENT A
           LEFT JOIN RESOURCE_T CR ON A.CREATED_BY = CR.ID
           LEFT JOIN OPERADOR CO   ON CR.EXTERNAL_CODE = CO.CDOPERADOR
           LEFT JOIN RESOURCE_T UR ON A.UPDATED_BY = UR.ID
           LEFT JOIN OPERADOR UO   ON UR.EXTERNAL_CODE = UO.CDOPERADOR
               WHERE A.SUBRELEASE = :SUBRELEASE_ID
                 AND A.LABEL LIKE ('%.sql'))
         WHERE 1 = 1
          $filters
SQL;

        $params = array (
            'SUBRELEASE_ID' => $subReleaseId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

	
}